import { Mx } from 'veho';
import { Table } from 'crostab';
import iconv from 'iconv-lite';

const parseCsv = (tx, {
  de = ',',
  qt = '"'
} = {}) => {
  let q = false,
      r = -1,
      c = -1,
      row = []; // q: true -> inside a quoted field

  const mx = [],
        ampCol = row => {
    if (!q) row[c] = row[c].trim();
    row[++c] = '';
  },
        ampRow = () => {
    mx[++r] = [];
    row = mx[r];
    c = 0;
    row[c] = '';
  };

  ampRow();

  for (let i = 0, ch, l = tx.length; i < l; i++) {
    ch = tx[i]; // current char

    if (q) {
      if (ch === qt) {
        if (tx[i + 1] === qt) {
          i++;
        } else {
          q = !q;
          continue;
        }
      }
    } else switch (ch) {
      case qt:
        // If it's just one 'quote', begin/end quoted field
        q = !q;
        continue;

      case de:
        // If it's a comma and we're not in a quoted field, move on to the next column
        ampCol(row);
        continue;

      case '\r':
        if (tx[i + 1] === '\n') ++i; // If it's a newline (CRLF) and we're not in a quoted field, skip the next char

        ampRow();
        continue;
      // and move on to the next row and move to column 0 of that new row

      case '\n':
        ampRow();
        continue;
    }

    row[c] += ch;
  }

  return mx;
};

const parseCsvEdge = (csvText, {
  de = ',',
  lf = '\r\n'
} = {}) => {
  return csvText.split(lf).map(row => row.split(de).map(word => word.trim()));
};

const popBlank = rows => {
  let {
    length: l
  } = rows,
      tail;

  while (l) {
    tail = rows[--l];
    if (tail.length && tail.some(x => x && x.trim().length)) break;
    rows.pop();
  }

  return rows;
};

class NaiveCsv {
  static toRows(csvText, {
    de = ',',
    lf = '\r\n',
    qt = '\"',
    transpose,
    decode,
    popBlank: popBlank$1 = false
  } = {}) {
    var _ref, _mx, _mx2, _mx3;

    if (decode) csvText = iconv.decode(csvText, decode);
    let mx = qt ? parseCsv(csvText, {
      de,
      qt
    }) : parseCsvEdge(csvText, {
      de,
      lf
    });
    if (transpose) mx = popBlank$1 ? (_ref = (_mx = mx, popBlank(_mx)), Mx.transpose(_ref)) : (_mx2 = mx, Mx.transpose(_mx2));
    return popBlank$1 ? (_mx3 = mx, popBlank(_mx3)) : mx;
  }

  static toTable(csvText, {
    de = ',',
    lf = '\r\n',
    qt,
    transpose,
    decode = false,
    popBlank,
    title
  } = {}) {
    const rows = NaiveCsv.toRows(csvText, {
      de,
      lf,
      qt,
      decode,
      transpose,
      popBlank
    }),
          head = rows.shift();
    return new Table(head, rows, title ? title : '');
  }

}

export { NaiveCsv };
